# Copyright (C) 2018 Allen Li
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""XDG Base Directory support

The standard directory paths are implemented using BaseDirs, but most
users can use module level constants defined at import time:

DATA_HOME
CONFIG_HOME
DATA_DIRS
CONFIG_DIRS
CACHE_HOME
RUNTIME_DIR
STATE_HOME

Note that STATE_HOME is not in the specification.

ensure() can be used to create directories if they do not exist in
compliance with the specification.
"""

__version__ = '1.0.0'

from pathlib import Path
import os


def ensure(dirpath: 'PathLike'):
    """Ensure directory exists according to XDG Base Directory spec."""
    os.makedirs(dirpath, mode=0o700, exist_ok=True)


class BaseDirs:

    """XDG Base Directory class.

    >>> b = BaseDirs({'HOME': '/home/alice'})
    >>> assert b.data_home == Path('/home/alice/.local/share')
    """

    def __init__(self, environ):
        home = Path(environ['HOME'])
        if 'XDG_DATA_HOME' in environ:
            self.data_home = Path(environ['XDG_DATA_HOME'])
        else:
            self.data_home = home / '.local/share'

        if 'XDG_CONFIG_HOME' in environ:
            self.config_home = Path(environ['XDG_CONFIG_HOME'])
        else:
            self.config_home = home / '.config'

        if 'XDG_DATA_DIRS' in environ:
            self.data_dirs = tuple(Path(p) for p in environ['XDG_DATA_DIRS'].split(':'))
        else:
            self.data_dirs = (Path('/usr/local/share'), Path('/usr/share'))

        if 'XDG_CONFIG_DIRS' in environ:
            self.config_dirs = tuple(Path(p) for p in environ['XDG_CONFIG_DIRS'].split(':'))
        else:
            self.config_dirs = (Path('/etc/xdg'),)

        if 'XDG_CACHE_HOME' in environ:
            self.cache_home = Path(environ['XDG_CACHE_HOME'])
        else:
            self.cache_home = home / '.cache'

        if 'XDG_RUNTIME_DIR' in environ:
            self.runtime_dir = Path(environ['XDG_RUNTIME_DIR'])
        else:
            self.runtime_dir = None

        # Not contained in specification
        if 'XDG_STATE_HOME' in environ:
            self.state_home = Path(environ['XDG_STATE_HOME'])
        else:
            self.state_home = home / '.local/state'


_dirs = BaseDirs(os.environ)
DATA_HOME = _dirs.data_home
CONFIG_HOME = _dirs.config_home
DATA_DIRS = _dirs.data_dirs
CONFIG_DIRS = _dirs.config_dirs
CACHE_HOME = _dirs.cache_home
RUNTIME_DIR = _dirs.runtime_dir
STATE_HOME = _dirs.state_home



# Copyright (C) 2017 Allen Li
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re
from setuptools import setup


def find_version(path):
    with open(path) as f:
        text = f.read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              text, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


setup(
    name='mir.xdg',
    version=find_version('mir/xdg/__init__.py'),
    description='XDG Base Directory support',
    long_description='',
    keywords='',
    url='https://github.com/darkfeline/mir.xdg',
    author='Allen Li',
    author_email='darkfeline@felesatra.moe',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3.6',
    ],

    packages=['mir.xdg'],
    python_requires='~=3.6',
    install_requires=[],
)



# Copyright (C) 2017 Allen Li
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
import stat

from mir import xdg


def test_ensure(tmpdir):
    tmpdir = Path(str(tmpdir))
    foo = tmpdir / 'foo'
    xdg.ensure(foo)
    assert foo.is_dir()
    assert stat.S_IMODE(foo.stat().st_mode) == 0o700


def test_data_home_from_environ():
    dirs = xdg.BaseDirs({'HOME': '/home/alice',
                         'XDG_DATA_HOME': '/foo'})
    assert dirs.data_home == Path('/foo')


def test_data_home_from_default():
    dirs = xdg.BaseDirs({'HOME': '/home/alice'})
    assert dirs.data_home == Path('/home/alice/.local/share')


def test_config_home_from_environ():
    dirs = xdg.BaseDirs({'HOME': '/home/alice',
                         'XDG_CONFIG_HOME': '/foo'})
    assert dirs.config_home == Path('/foo')


def test_config_home_from_default():
    dirs = xdg.BaseDirs({'HOME': '/home/alice'})
    assert dirs.config_home == Path('/home/alice/.config')


def test_data_dirs_from_environ_single():
    dirs = xdg.BaseDirs({'HOME': '/home/alice',
                         'XDG_DATA_DIRS': '/foo'})
    assert dirs.data_dirs == (Path('/foo'),)


def test_data_dirs_from_environ_many():
    dirs = xdg.BaseDirs({'HOME': '/home/alice',
                         'XDG_DATA_DIRS': '/foo:/bar'})
    assert dirs.data_dirs == (Path('/foo'), Path('/bar'))


def test_data_dirs_from_default():
    dirs = xdg.BaseDirs({'HOME': '/home/alice'})
    assert dirs.data_dirs == (Path('/usr/local/share'), Path('/usr/share'))


def test_config_dirs_from_environ_single():
    dirs = xdg.BaseDirs({'HOME': '/home/alice',
                         'XDG_CONFIG_DIRS': '/foo'})
    assert dirs.config_dirs == (Path('/foo'),)


def test_config_dirs_from_environ_many():
    dirs = xdg.BaseDirs({'HOME': '/home/alice',
                         'XDG_CONFIG_DIRS': '/foo:/bar'})
    assert dirs.config_dirs == (Path('/foo'), Path('/bar'))


def test_config_dirs_from_default():
    dirs = xdg.BaseDirs({'HOME': '/home/alice'})
    assert dirs.config_dirs == (Path('/etc/xdg'),)


def test_runtime_dir_from_environ():
    dirs = xdg.BaseDirs({'HOME': '/home/alice',
                         'XDG_RUNTIME_DIR': '/foo'})
    assert dirs.runtime_dir == Path('/foo')


def test_runtime_dir_from_default():
    dirs = xdg.BaseDirs({'HOME': '/home/alice'})
    assert dirs.runtime_dir is None


def test_state_home_from_environ():
    dirs = xdg.BaseDirs({'HOME': '/home/alice',
                         'XDG_STATE_HOME': '/foo'})
    assert dirs.state_home == Path('/foo')


def test_state_home_from_default():
    dirs = xdg.BaseDirs({'HOME': '/home/alice'})
    assert dirs.state_home == Path('/home/alice/.local/state')
